    
' eschecs.bas

#include once ".\libraries\mdlanguage\mdlanguage.bi"
#include once ".\libraries\sgui\sgui.bas"
#include once ".\libraries\sgui\simplegadget.bas"
#include once ".\libraries\sgui\clickmapgadget.bas"
#include once ".\libraries\sgui\togglegadget.bas"
#include once ".\libraries\sgui\radiobutton.bas"
#include once ".\libraries\iniobject\inifile.bi"
#include once ".\libraries\colors\colors.bi"

#include once ".\types.bas"
#include once ".\chess.bas"
#include once ".\files.bas"
#include once ".\book.bas"

#define appPath ExePath & "\"
#define resPath ".\bitmap\"
#define buttonHeight 28
#define buttonWidth 158
#define espacement 4 

#ifndef false
#define false 0
#define true (not false)
#endif

extern "C" lib ".\engine"
  declare function EngineMove stdcall alias "EngineMove" ( _
    byval positionData as string = "", _
    byval logName as string = "" _
  ) as string
end extern

extern "C" lib ".\validator"
  declare function IsFEN stdcall alias "IsFEN" (byval s as string) as integer
end extern

using sGUI

const programName = "ESCHECS"
const version = "0.9.1"
const optionsFile = "options.ini"

declare sub Info(byval as string = "")
declare sub InfoSleep(byval as string)
declare function IsValidStartingSquare(byref as chess.tChessGame, byval as string) as tBoolean
declare function IsPromotionMove(byref as chess.tChessGame, byval as string) as tBoolean
declare function IsCastlingMove(byref as chess.tChessGame, byval as string) as tBoolean
declare function IsEnPassantMove(byref as chess.tChessGame, byval as string) as tBoolean
declare sub LoadOptions(byref as string)
declare function ReplaceFieldName(byval as string) as string

screenres 336, 436 + 20 + 8, 32,, 4
windowtitle programName
const textHeight = 14
dim as integer breit, hoch
screeninfo breit, hoch
width breit \ 8, hoch \ textHeight

dim shared as uinteger infoTextColor
dim shared as uinteger infoColor

#include once ".\colors.bas"
#include once ".\pieces.bas"
#define boardBkColor White

graph.boardLeft = 8
graph.boardTop = 8

InitGFX
Info()

dim language as string
LoadOptions(language)
LoadCustomLanguage(default, language, "language\")
dim sNew as string = GetTranslation(default, "new")
dim sFlip as string = GetTranslation(default, "flip")
dim sQuit as string = GetTranslation(default, "quit")
dim sCheck as string = GetTranslation(default, "check")
dim sWhiteToMove as string = GetTranslation(default, "white")
dim sBlackToMove as string = GetTranslation(default, "black")
dim sAutoplay as string = GetTranslation(default, "autoplay")
dim sMove as string = GetTranslation(default, "move")
dim sState(gsCheckmate to gsDraw) as string
sState(gsCheckmate) = GetTranslation(default, StateToStr(gsCheckmate))
sState(gsStalemate) = GetTranslation(default, StateToStr(gsStalemate))
sState(gsDraw) = GetTranslation(default, StateToStr(gsDraw))
dim sWait as string = GetTranslation(default, "wait")

dim as EventHandle ptr event = new EventHandle
dim as gadget ptr button1, button2, button3, button4
dim as gadget ptr boardClickSensor
dim as gadget ptr toggle1
dim as gadget ptr radioa, radiob, radioc, radiod
dim as integer field_ID

type chessfield 
  fieldname as string 
  selected as integer 
  posx as integer 
  posy as integer 
end type

dim as chessfield Board(1 to 64)

for k as integer = 0 to 7 
  Board(1 + k * 8).fieldname = "a" & str(k + 1) 
  Board(2 + k * 8).fieldname = "b" & str(k + 1) 
  Board(3 + k * 8).fieldname = "c" & str(k + 1) 
  Board(4 + k * 8).fieldname = "d" & str(k + 1) 
  Board(5 + k * 8).fieldname = "e" & str(k + 1) 
  Board(6 + k * 8).fieldname = "f" & str(k + 1) 
  Board(7 + k * 8).fieldname = "g" & str(k + 1) 
  Board(8 + k * 8).fieldname = "h" & str(k + 1) 
next k

dim as byte autoPlay = false
dim as integer autoColor = chess.cBlack

button1 = AddSimpleGadget(event, 170, 336 + 0 * (buttonHeight + espacement), buttonWidth, buttonHeight, sNew)
button2 = AddSimpleGadget(event, 008, 336 + 2 * (buttonHeight + espacement), buttonWidth, buttonHeight, sFlip)
button3 = AddSimpleGadget(event, 170, 336 + 2 * (buttonHeight + espacement), buttonWidth, buttonHeight, sQuit)
button4 = AddSimpleGadget(event, 170, 336 + 1 * (buttonHeight + espacement), buttonWidth, buttonHeight, sMove)
boardClickSensor = AddClickMapGadget(event, graph.boardLeft, graph.boardTop, "libraries\sgui\ClickMap.bmp")
toggle1 = AddToggleGadget(event, 008, 336 + 1 * (buttonHeight + espacement), buttonWidth, buttonHeight, autoPlay, sAutoplay)
radioa  = AddRadioButton(event, 008 + 0 * buttonWidth \ 2, 436 + 00, 0, "Knight", 0)
radiob  = AddRadioButton(event, 008 + 1 * buttonWidth \ 2, 436 + 00, 0, "Bishop", radioa)
radioc  = AddRadioButton(event, 170 + 0 * buttonWidth \ 2, 436 + 00, 0, "Rook", radioa)
radiod  = AddRadioButton(event, 170 + 1 * buttonWidth \ 2, 436 + 00, 1, "Queen", radioa)

GadgetOn(button1, radiod)

graph.upsideDown = false

line (graph.boardLeft, graph.boardTop)-(graph.boardLeft + 320, graph.boardTop + 320), boardBkColor, BF

dim startingPosition as string

if IsFEN(Command(1)) then
  StartingPosition = Command(1)
else
  StartingPosition = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"
end if

dim set as graph.T3

set.Create()
set.ChangePieces(mid(StartingPosition, 1, instr(StartingPosition, " ") - 1))
set.RedrawPieces()

dim game as chess.tChessGame
dim move as string = ""
dim state as tGameState = gsCheckmate
dim check as tBoolean = isFalse
dim bookMoveAvailable as integer
dim history as tFileOfString
dim journal as tFileOfString

history.Initialize(appPath & "history.fen")
journal.Initialize(appPath & "journal.txt")
journal.AddLine(programName & " " & version)
journal.AddLine("FreeBASIC Compiler " & __FB_VERSION__)
journal.AddLine(Date & " " & Time)

#macro UpdateAll()
game.GenerateGeometricalMoves()
game.GenerateSpecialMoves()
game.CheckMovesLegality()
state = game.GameState
if state = gsInProgress then
  with history
    if .Found(.LastLine()) = 3 then state = gsDraw
  end with
end if
if len(move) = 4 then
  set.MovePiece(move)
end if
if state = gsInProgress then
  check = game.Check
  if check then InfoSleep(sCheck)
  Info(iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
else
  Info(sState(state))
end if
#endmacro

dim bookLoaded as byte = (LoadBook() = 0)
dim promotion as byte
dim sPromotion as string = "wq"
dim oneMove as byte = false

game.Initialize(StartingPosition)
history.AddLine(game.FEN)
UpdateAll()

do
  event->xSleep(1)
  if state = gsInProgress then
    if oneMove or (autoPlay andalso (autoColor = game.activeColor)) then
      if oneMove then oneMove = false 
      bookMoveAvailable = false
      if BookLoaded then
        move = BookMove(game.EPD)
        if (len(move) = 4) andalso game.IsLegal(move) then
          bookMoveAvailable = true
          
          if IsPromotionMove(game, move) then
            promotion = chess.cQueen
            sPromotion[0] = iif(game.ActiveColor = chess.cWhite, asc("w"), asc("b"))
            sPromotion[1] = asc("q")
            game.DoMove(move, promotion)
            set.Promote(left(move, 2), sPromotion)
          else
            if IsCastlingMove(game, move) then
              select case move
                case "e1g1" : set.MovePiece("h1f1")
                case "e1c1" : set.MovePiece("a1d1")
                case "e8g8" : set.MovePiece("h8f8")
                case "e8c8" : set.MovePiece("a8d8")
              end select
            else
              if IsEnPassantMove(game, move) then
                set.HidePiece(chr(move[2], move[1]))
              end if
            end if
          end if
          
          game.DoMove(move)
          journal.AddLine(move & " book")
          history.AddLine(game.FEN)
          UpdateAll()
        else
          rem
        end if
        move = ""
      end if
      if bookMoveAvailable = false then
        Info(sWait)
        
        move = EngineMove(game.FEN)
        rem move = EngineMove(game.FEN, "c:\atelier\basic\eschecs\debug.txt")
        
        if game.IsLegal(move) then
        
          if IsPromotionMove(game, move) then
            promotion = chess.cQueen
            sPromotion[0] = iif(game.ActiveColor = chess.cWhite, asc("w"), asc("b"))
            sPromotion[1] = asc("q")
            game.DoMove(move, promotion)
            set.Promote(left(move, 2), sPromotion)
          else
            if IsCastlingMove(game, move) then
              select case move
                case "e1g1" : set.MovePiece("h1f1")
                case "e1c1" : set.MovePiece("a1d1")
                case "e8g8" : set.MovePiece("h8f8")
                case "e8c8" : set.MovePiece("a8d8")
              end select
            else
              if IsEnPassantMove(game, move) then
                set.HidePiece(chr(move[2], move[1]))
              end if
            end if
            
            game.DoMove(move)
          end if
          journal.AddLine(move & " engine")
          history.AddLine(game.FEN) 
          UpdateAll()
        else
          journal.AddLine(move & " engine illegal move")
          autoPlay = false
          if GetSelect(toggle1) then SetSelect(toggle1, 0)
          Info(iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
        end if
        move = ""
      end if
    end if
  end if
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case button1
        set.ClearBoard()
        set.ChangePieces()
        set.RedrawPieces()
        
        game.Initialize()
        history.Initialize()
        history.AddLine(game.FEN)
        UpdateAll()
        
        journal.AddLine("new game")
        
      case button2
        set.ClearBoard()
        graph.upsideDown = not graph.upsideDown
        for i as integer = 0 to set.number - 1
          set.array(i).img.fSX = graph.ScrX(set.array(i).fX)
          set.array(i).img.fSY = graph.ScrY(set.array(i).fY)
        next i
        set.RedrawPieces()
        
      case button3
        event->EXITEVENT = 1
      
      case button4
        if autoPlay then
          autoColor = chess.cBlack * autoColor
        else
          oneMove = true
        end if
        
      case boardClickSensor
        field_ID = getClickMapColor(boardClickSensor)
        
        if state = gsInProgress then
          if graph.upsideDown then
            move += ReplaceFieldName(Board(field_ID).fieldname)
          else
            move += Board(field_ID).fieldname
          end if
        end if
        
        if len(move) = 2 then
          if IsValidStartingSquare(game, move) then
            Info()
          else
            beep
            Info(iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
            move = ""
          end if
        end if
        
        if len(move) = 4 then
          if game.IsLegal(move) then
            if IsPromotionMove(game, move) then
              
              if GetSelect(radioa) then
                promotion = chess.cKnight
              elseif GetSelect(radiob) then
                promotion = chess.cBishop
              elseif GetSelect(radioc) then
                promotion = chess.cRook
              elseif GetSelect(radiod) then
                promotion = chess.cQueen
              end if
              
              sPromotion[0] = iif(game.ActiveColor = chess.cWhite, asc("w"), asc("b"))
              select case as const promotion
                case chess.cKnight : sPromotion[1] = asc("n")
                case chess.cBishop : sPromotion[1] = asc("b")
                case chess.cRook   : sPromotion[1] = asc("r")
                case chess.cQueen  : sPromotion[1] = asc("q")
              end select
              
              game.DoMove(move, promotion)
              set.Promote(left(move, 2), sPromotion)
              
            else
              if IsCastlingMove(game, move) then
                select case move
                  case "e1g1" : set.MovePiece("h1f1")
                  case "e1c1" : set.MovePiece("a1d1")
                  case "e8g8" : set.MovePiece("h8f8")
                  case "e8c8" : set.MovePiece("a8d8")
                end select
              else
                if IsEnPassantMove(game, move) then
                  set.HidePiece(chr(move[2], move[1]))
                end if
              end if
              game.DoMove(move)
            end if
            journal.AddLine(move & " user")
            history.AddLine(game.FEN)
            UpdateAll()
          else
            Info(iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
          end if
          move = ""
        end if
      case toggle1
        autoPlay = GetSelect(toggle1)
    end select
  end if
loop until event->EXITEVENT

delete event
if bookLoaded then CloseBook()
set.Free

journal.AddLine("exit")
end

sub Info(byval s as string = "")
  const x = 8
  const y = 336
  FillA(x, y, buttonWidth, buttonHeight, infoColor, 1)
  draw string _
    ((buttonWidth - 8 * len(s)) \ 2 + x, y + (buttonHeight - textHeight) \ 2), _
    left(s, 18), _
    infoTextColor
end sub

sub InfoSleep(byval s as string)
  Info(s)
  sleep 1000
end sub

function IsValidStartingSquare(byref g as chess.tChessGame, byval m as string) as tBoolean
  return sgn(g.board(m[0] - 96, m[1] - 48)) = g.activeColor
end function

function IsPromotionMove(byref g as chess.tChessGame, byval m as string) as tBoolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

function IsCastlingMove(byref g as chess.tChessGame, byval m as string) as tBoolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cKing) andalso (Abs(m[0] - m[2]) = 2)
end function

function IsEnPassantMove(byref g as chess.tChessGame, byval m as string) as tBoolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((abs(g.board(m[2] - 96, m[3] - 48)) = chess.cNothing) and (m[0] <> m[2]))
end function

sub LoadOptions(byref aLanguage as string)
  dim myIni as inifobj.iniobj = inifobj.iniobj(optionsFile)
  dim update as integer = false
  var s = myIni.ReadString("interface", "language", "")
  select case s
  	case "de"
  		aLanguage = "de"
  	case "en"
  		aLanguage = "en"
  	case "fr"
  		aLanguage = "fr"
  	case else
      myIni.WriteString("interface", "language", "en")
  		aLanguage = "en"
  		update = true
  end select
  if update then myIni.UpdateFile()
end sub

function ReplaceFieldName(byval s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  return chr(asc1 + 96, asc2 + 48)
end function
